#define INTEGER 257
#define IDENTIFIER 258
#define FLOAT 259
#define TERMINAL 260
#define NETWORK 261
#define EXTERNSPEC 262
#define NET 263
#define NENH 264
#define PENH 265
#define NDEP 266
#define RES 267
#define CAP 268
#define SEMICOLON 269
#define COLON 270
#define COMMA 271
#define PERIOD 272
#define LPS 273
#define RPS 274
#define LCB 275
#define RCB 276
#define LSB 277
#define RSB 278
#define T_WIDTH 279
#define T_LENGTH 280
#define T_RISE 281
#define T_FALL 282
#define ATSIGN 283
#define ASSIGN 284
#define DOTDOT 285
typedef union {
	int   ival;
	char *sval;
	char cval;
	int *pival;
	float fval;
	double dval;
	} YYSTYPE;
extern YYSTYPE yylval;
