static char *SccsId = "@(#)dmperror.c 3.5 (TU-Delft) 03/26/91";
/**********************************************************

Name/Version      : The Design Data Management Project

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000/S500

Author(s)         : S. de Graaf
                    A. van der Hoeven
                    T.G.R. van Leuken
                    N. van der Meijs
		    T. Vogel
		    P. van der Wolf
Creation date     : 04-Dec-1986
Modified by       : S. de Graaf
Modification date : 07-Apr-1988


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1985-1988, All rights reserved
**********************************************************/
#include "dmstd.h"

dmPerror (s)
char   *s;
{
    if (s && *s != '\0')
	fprintf (stderr, "%s: ", s);

    if (dmerrno > 0 && dmerrno <= dmnerr)
	fprintf (stderr, "%s\n", dmerrlist[dmerrno]);
    else
	fprintf (stderr, "DMI error number: %d\n", dmerrno);
}
