
static char *SccsId = "@(#)dmputesc.c 3.2 (TU-Delft) 03/26/91";
/**********************************************************

Name/Version      : The Design Data Management Project

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000/S500

Author(s)         : S. de Graaf
                    A. van der Hoeven
                    T.G.R. van Leuken
                    N. van der Meijs
		    T. Vogel
		    P. van der Wolf
Creation date     : 14-Aug-1987
Modified by       :
Modification date :


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1985-1987 , All rights reserved
**********************************************************/
#include "dmstd.h"

dmPutDataEscape (dmfile, fmt)
DM_STREAM * dmfile;
int     fmt;
{
    char s[256];

    _dmSprintf (s, "dmPutDesignData: %d", fmt);
    dmerrno = DME_FMT;
    dmError (s);
    return (-1);
}
