/* SccsId = "@(#)extern.h 4.7 (TU-Delft) 12/09/92" */
/**********************************************************

Name/Version      : simeye/4.7

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : A.J. van Genderen
Creation date     : 29-May-1990
Modified by       : 
Modification date : 


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1990 , All rights reserved
**********************************************************/
extern struct signal *Begin_signal;
extern struct signal *End_signal;
extern int Nr_signals;

extern long Begintime;
extern long Endtime;
extern long SimEndtime;
extern double Timescaling;
extern double Voltscaling;
extern int Global_umin;
extern int Global_umax;

extern int Logic;
extern int currLogic;
extern int Spice;
extern int Cmd;
extern int SimLevel;
extern int doDetailZoom;
extern int tryNonCapital;
extern int usedNonCapital;

extern struct signal *Top_signal;
extern struct signal *Bottom_signal;
extern int Curr_nr_signals;

extern long startTime;
extern long stopTime;

extern int curr_umin;
extern int curr_umax;

extern int Append;

extern int editing;
extern int somethingChanged;

extern int use_io_views;
