/* SccsId = "@(#)odb.h 4.2 (TU-Delft) 06/11/91" */
/**********************************************************

Name/Version      : sls_mkdb/4.2

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : A.C. de Graaf
Creation date     : 10-Jul-1986
Modified by       : S. de Graaf
Modification date : 10-Jul-1986


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1986 , All rights reserved
**********************************************************/
#define	PBLKSIZ	1024
#define	DBLKSIZ	1024
#define	BYTESIZ	8

typedef	struct datum
{
	char	*dptr;
	int	dsize;
} datum;

typedef struct db_files {
	char * dbname;
	int DbRdOnly;
	int DirF, dir_b, dir_t; 
	int PagF, pag_b, pag_t; 
	
	long	bitno;
	long	maxbno;
	long	blkno;
	long	hmask;

	char	pagbuf[PBLKSIZ];
	char	dirbuf[DBLKSIZ];
} DbAcces;

DbAcces * 	dbOpen();
datum		dbfetch();
datum		makdatum();
datum		firstkey();
datum		nextkey();
datum		firsthash();
long		calchash();
long		hashinc();
