/* SccsId = "@(#)netelem.h 4.3 (TU-Delft) 11/25/92" */
/**********************************************************

Name/Version      : sls_mkdb/4.3

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : A.C. de Graaf
Creation date     : 10-Jul-1986
Modified by       : S. de Graaf
Modification date : 10-Jul-1986


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1986 , All rights reserved
**********************************************************/
#define  N_NET       0
#define  N_TERMINAL  1
#define  N_GLOBADDED 2

class simpleNet {
public:
	class netMember	*nmem;
	class simpleNet *next;
	class simpleNet *prev;
};

class netMember {
public:
	NetworkInstance *inst;
	class netelem   *net;
	class netMember *next;
	class simpleNet *snet;
};

class netelem : public link {
	void		new(char *, class stack *, char);
	void		delete(void);
public:
	char 		*name;
	int             type;
	Stack		*xs;
	Queue		*eqv;
	class netMember	*nmem;
	void		print();
	void		to_db();
};

class net_ref : public link {
	void		new(class netelem *, class stack *);
	void		delete(void);
public:
	class netelem	*net;
	Stack		*net_xs;
	NetworkInstance *inst;
	Stack		*inst_xs;
	Stack		*ref_xs;
};

typedef class netMember Nmem;
