static char *SccsId = "@(#)assert.c 4.3 (TU-Delft) 01/28/93";
/**********************************************************

Name/Version      : aux/4.3

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Authors           : A.J. van Genderen
                  : N.P. van der Meijs
Creation date     : 15-Mar-1988
Modified by       :
Modification date :


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1988. All rights reserved.
**********************************************************/

/*
 * assert - print an 'assertion failed' message,
 * by calling the say function.
 * indicating file and line of failure.
 * This message is unconditional, the test is performed
 * by an ASSERT macro.
 */
assert (file, line)
char * file;
int    line;
{
    say ("%s, %d: assertion failed", file, line);
}
