static char *SccsId = "@(#)prFoot.c 4.2 (TU-Delft) 03/20/91";
/**********************************************************

Name/Version      : xsls/4.2

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : A.J. van Genderen
Creation date     : 16-Feb-1987
Modified by       : A.J. van Genderen
Modification date : 16-Feb-1987


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1987 , All rights reserved
**********************************************************/
#include "incl.h"

extern int out_indent;
extern int language;

prFoot (ntw, submod)
struct model_info *ntw;
int submod;
{
    out_indent = 0;

    switch (language) {
	case SLS:
	    oprint (0, "}");
	    break;
	case SPICE:
	    if (submod) {
		oprint (0, ".ends ");
	    }
	    else {
		oprint (0, "* end ");
	    }
	    oprint (1, ntw -> name);
	    break;
	case SIMON:
	    if (submod) {
		oprint (0, "   mend ");
	    }
	    else {
		oprint (0, "   netend ");
	    }
	    oprint (1, ntw -> name);
	    break;
	case EDIF:
	     oprint (0, ")");
	     oprint (0, ")");
	     break;
    }

    oprint (0, "\n");
}
