/* SccsId = "@(#)libc.h 4.4 (TU-Delft) 05/04/93" */
/**********************************************************

Name/Version      : aux/4.4

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Authors           : A.J. van Genderen
                  : N.P. van der Meijs
Creation date     : 20-Nov-92
Modified by       :
Modification date :


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1992. All rights reserved.
**********************************************************/

/*
 * Next are declarations of libc functions
 */
extern char * strtok ();
extern char * strcpy ();
extern char * strncpy ();
extern char * strcat ();
extern char * strrchr ();
extern char * rindex ();
extern char * mktemp ();

#ifdef linux
#include <stdio.h>
#else
#ifdef BSD
extern int  sleep ();
#ifndef apollo
extern char * sprintf ();
#endif
#else /* SYSV */
extern unsigned int sleep ();
extern int sprintf ();
#endif /* SYSV */
#endif /* linux */

