/* SccsId = "@(#)transistor.h 4.3 (TU-Delft) 11/24/92" */
/**********************************************************

Name/Version      : space/4.3

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : A.J. van Genderen
		  : N.P. van der Meijs
Creation date     : 15-Mar-1988
Modified by       :
Modification date :


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1988. All rights reserved.
**********************************************************/
typedef struct transistor {
    float totPerimeter;
    float dsPerimeter;
    float surface;
    coor_t xl;
    coor_t yb;
    struct elemDef * type;
    char name[8];
    struct nodeTorLink * gate;
    struct nodeTorLink * drains;
    struct tile * subs;
    struct dsBoundary * boundaries;
    struct transTileInfo * tileInfo;
} transistor_t;

typedef struct dsBoundary {
    struct dsCoor * points;  
    struct subnode * dsCond;
    struct dsBoundary * next;
} dsBoundary_t;

typedef struct dsCoor {
    coor_t x;
    coor_t y;
    int nr;
    struct dsCoor * next;
} dsCoor_t;

typedef struct transTileInfo {
    struct tileRef * tiles;
} transTileInfo_t;

