/* SccsId = "@(#)extern.h 3.1 (TU-Delft) 06/08/90" */
/**********************************************************

Name/Version      : nspice/3.1

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : A.J. van Genderen
Creation date     : 8-June-1990
Modified by       :
Modification date :


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1990 , All rights reserved
**********************************************************/
extern struct signal *Begin_signal;
extern struct signal *End_signal;
extern int Nr_signals;

extern long Endtime;
extern double Timescaling;

extern struct node_ref *Begin_print;
extern struct node_ref *End_print;

extern struct node_ref *Begin_table;
extern struct node_ref *End_table;
