/* SccsId = "@(#)plot.h 4.3 (TU-Delft) 01/28/93" */
/**********************************************************

Name/Version      : aux/4.3

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Authors           : A.J. van Genderen
                  : N.P. van der Meijs
Creation date     : 15-Mar-1988
Modified by       :
Modification date :


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1988. All rights reserved.
**********************************************************/
#define PLOT_KEY FILE

#define SOLID       	0
#define DOTTED      	1
#define DASHED      	2

#define CENTER   	0
#define ABOVE		1
#define BELOW		2
#define LJUST		3
#define RJUST		4
