static char *SccsId = "@(#)proc_cont.c 4.1 (TU-Delft) 05/24/90";
/**********************************************************

Name/Version      : cldm/4.1

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : J. Annevelink
Creation date     : 18-May-1983
Modified by       : S. de Graaf
Modification date : 26-Jan-1987


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1987 , All rights reserved
**********************************************************/
#include "extern.h"

proc_cont (dir, width)
int  dir, width;
{
    if (w_dir == dir || dir == 0) {
    /* change of direction, update wire start position */
	if (int_val[0] > 0) {
	    if (w_dir == -1)	/* next box in pos x_direction */
		w_x = w_x - w_width;
	    else		/* next box in pos y-direction */
		w_y = w_y - w_width;
	    int_val[0] = int_val[0] + w_width;
	}
	else {
	    if (w_dir == -1)	/* next box in neg x-direction */
		w_x = w_x + w_width;
	    else		/* next box in neg y-direction */
		w_y = w_y + w_width;
	    int_val[0] = int_val[0] - w_width;
	}
    }

    if (dir) w_dir = dir;

    w_width = width;

    proc_wire ();
}
