static char *SccsId = "@(#)fr_errstr.c 4.1 (TU-Delft) 05/24/90";
/**********************************************************

Name/Version      : DUBCHECK/4.1

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)	  : J. Liedorp
Creation date	  : 25-Nov-1985
Modified by       : S. de Graaf
Modification date : 09-Dec-1985


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1984 , All rights reserved
**********************************************************/
#include "dubcheck.h"

rmv_errstr () {

/* This procedure frees the structure containing the	 */
/* equivalence of the groups of mask1 and mask2.	 */

    unsigned    i;
    unsigned    j;
    struct group   *g_pntr;
    struct eq  *eq_pntr;

    for (i = 0; i < GRP_BUFLEN; i++) {
	if (group_arr[i] != NULL) {
	    g_pntr = group_arr[i];
	    for (j = 0; j < GRP_BUFLEN; j++) {
		eq_pntr = g_pntr[i].eq_pntr;
		while (eq_pntr != NULL) {
		    FREE (eq_pntr);
		    eq_pntr = eq_pntr -> next;
		}
	    }
	    FREE (g_pntr);
	    group_arr[i] = NULL;
	}
    }
}

free_errs (err_head)
struct chk_err *err_head;
{

/* this procedure frees the error linked_list		 */

    struct chk_err *c_err;
    c_err = err_head;
    while (c_err != NULL) {
	cfree (c_err);
	c_err = c_err -> next;
    }
}
