static char *SccsId = "@(#)files.c 4.3 (TU-Delft) 06/18/90";
/**********************************************************

Name/Version      : cldm/4.3

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : J. Annevelink
Creation date     : 18-May-1983
Modified by       : S. de Graaf
Modification date : 26-Jan-1987
Modification date : 20-May-1988


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1987-1988, All rights reserved
**********************************************************/
#include "extern.h"

open_files ()
{
    fp_box  = dmOpenStream (mod_key, "box" , "w");
    fp_info = dmOpenStream (mod_key, "info", "w");
    fp_mc   = dmOpenStream (mod_key, "mc"  , "w");
    fp_nor  = dmOpenStream (mod_key, "nor" , "w");
    fp_term = dmOpenStream (mod_key, "term", "w");
}

close_files ()
{
    dmCloseStream (fp_box, COMPLETE);
    dmCloseStream (fp_info, COMPLETE);
    dmCloseStream (fp_mc, COMPLETE);
    dmCloseStream (fp_nor, COMPLETE);
    dmCloseStream (fp_term, COMPLETE);
}
