/* SccsId = "@(#)func.h 4.1 (TU-Delft) 05/24/90" */

#define	BSCOPY	strcpy
#define	BSNCOPY	strncpy
#define	BSCMP	strcmp
#define	BSNCMP	strncmp
#define	BSLEN	strlen
#define	BTTRUE	'I'
#define	BTFALSE	'O'
#define	BTUNDEF	'X'
#define	BTFREE	'F'

extern float cap_val();
extern float statcap_val();
extern float dyncap_val();

extern char BTAND ();
extern char BTEXNOR ();
extern char BTEXOR ();
extern char BTINVERT ();
extern char BTNAND ();
extern char BTNOR ();
extern char BTOR ();

extern BSFREE ();
extern BSRESET ();
extern BSSET ();
extern BSUNDEF ();
extern char * BSROTATE ();
extern int BSTOI ();

extern char * BWAND ();
extern char * BWEXNOR ();
extern char * BWEXOR ();
extern char * BWINVERT ();
extern char * BWNAND ();
extern char * BWNOR ();
extern char * BWOR ();

extern char * ITOBS ();
extern char * ITOTC ();
extern int TCTOI ();

extern curs_error ();
extern func_error ();
extern single_curs_step ();
extern single_step ();
