static char *SccsId = "@(#)fr_frmstr.c 4.1 (TU-Delft) 05/24/90";
/**********************************************************

Name/Version      : NBOOL/4.1

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : J. Liedorp
Creation date     : 31-Jul-1985
Modified by       : S. de Graaf
Modification date : 02-Dec-1985


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1985 , All rights reserved
**********************************************************/
#include "nbool.h"

free_formstruct () {
/* This procedure frees the form_structure made by	 */
/* mk_frmstrct						 */

    int     i;			/* loop variable	 */
    struct form *frm_pntr;	/* ptr to form struct.	 */
    struct min_term *mt_pntr;	/* ptr to min_term struct */

    frm_pntr = fp_head;
    while (frm_pntr != NULL) {
	for (i = 0; i < BUFLEN; i++) {
	    FREE (frm_pntr -> b_pntr[i]);
	}
	mt_pntr = frm_pntr -> mt_pntr;
	while (mt_pntr != NULL) {
	    FREE (mt_pntr);
	    mt_pntr = mt_pntr -> next;
	}
	FREE (frm_pntr);
	frm_pntr = frm_pntr -> next;
    }
}
