/* SccsId = "@(#)type.h 4.1 (TU-Delft) 05/23/90" */
/**********************************************************

Name/Version      : sls_exp/4.1

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : A.J. van Genderen
Creation date     : 10-Jul-1986
Modified by       : S. de Graaf
Modification date : 10-Jul-1986


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1986 , All rights reserved
**********************************************************/

typedef struct path_spec
{
    char name[NAMESIZE];
    int xarray[MAXDIM+1][2];
    struct path_spec * next;
    struct path_spec * also;
} PATH_SPEC;

typedef struct node_ref_list
{
    int nx;
    int * xptr;
    struct node_ref_list * next;
} NODE_REF_LIST ;

typedef struct call_list {
    char inst_name[NAMESIZE];
    int xarray[MAXDIM+1][2];
    int mcx;
    int number;
    struct call_list * next;
} CALL_LIST;

typedef struct child_list {
    char object[NAMESIZE];
    short imported;
    CALL_LIST * calls;
    struct child_list * next;
} CHILD_LIST;
