static char *SccsId = "@(#)dmfatal.c 3.4 (TU-Delft) 03/26/91";
/**********************************************************

Name/Version      : The Design Data Management Project

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000/S500

Author(s)         : S. de Graaf
                    A. van der Hoeven
                    T.G.R. van Leuken
                    N. van der Meijs
		    T. Vogel
		    P. van der Wolf
Creation date     : 04-Dec-1986
Modified by       : 
Modification date : 


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1985-1987 , All rights reserved
**********************************************************/
#include "dmstd.h"

_dmFatal (format, arg1, arg2)
char   *format,
       *arg1,
       *arg2;
{
    fprintf (stderr, "fatal error in function ");
    fprintf (stderr, format, arg1, arg2);
    fprintf (stderr, "\n");
    dmQuit ();
    exit (2);
}
