/* SccsId = "@(#)extern.h 4.4 (TU-Delft) 04/20/93" */
/**********************************************************

Name/Version      : makebox/4.4

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : S. de Graaf
Creation date     : 22-Apr-1986
Modified by       : S. de Graaf
Modification date : 05-Jun-1987


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1987 , All rights reserved
**********************************************************/
#include "define.h"

extern struct clist *celllist;
extern struct tmtx  *tm_p, *tm_s;

extern DM_PROCDATA *process;
extern DM_STREAM  **fp_bxx;
extern DM_STREAM  **fp_nxx;
extern DM_STREAM   *fp_tid;
extern DM_STREAM   *fp_spec;
extern DM_STREAM   *fp_tidpos;
extern DM_CELL     *cellkey;
extern DM_PROJECT  *project;
extern double      *px, *py;
extern double       rad01, rad90;

extern char   fn_exp[];
extern char   buf[];
extern char  *cellname;
extern char  *argv0;

extern long  *no_bxx;
extern long   exp_reg[];
extern int    part_exp;
extern int    dflag;
extern long   exp_depth;
extern long   term_no;
extern int    mask_no;
extern int    t_mask_no;
extern int    level;
extern int    s_mode;
extern int    samples;
extern int    pt_size;
extern int    Lflag;
extern int    nordatahack;
extern int    hier_mode;
extern int    netterm;
extern int    add3Dbox;
extern int    noTidpos;
extern int    two_levels;
extern int    noErrMes;

extern char * imageName;
