static char *SccsId = "@(#)BTAND.c 4.1 (TU-Delft) 05/24/90";
/**********************************************************

Name/Version      : libfunc/4.1

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : GOULD

Author(s)         : O. Hol
Creation date     : 18-Feb-1988
Modified by       : 
Modification date : 


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1988 , All rights reserved
**********************************************************/
#include <stdio.h>
#include <varargs.h>


/* BTAND : returns result of and-operation of given bits    */

char BTAND (va_alist)
va_dcl
{
    va_list	ap;
    char	tmps;
    char	res;

    va_start(ap);
    res= 'I';
    while ( (tmps = (char)va_arg(ap,int) ) != '@' && res!='O' ) {
	if ( tmps == 'O' )
	    res = 'O';
	else if ( tmps == 'X' )
	    res = 'X';
    }
    va_end(ap);
    return(res);
}
