/* SccsId = "@(#)proto.h 4.5 (TU-Delft) 04/20/93" */
/**********************************************************

Name/Version      : aux/4.5

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Authors           : A.J. van Genderen
                  : N.P. van der Meijs
Creation date     : 20-Nov-92
Modified by       :
Modification date :


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
         The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1992. All rights reserved.
**********************************************************/

/* from alarm.c */
extern setAlarmInterval ();
extern void catcher ();

/* from assert.c */
extern assert ();

/* from clock.c */
extern void clockInit ();
extern void tick ();
extern void tock ();
extern void clockPrintAll ();
extern void clockPrintTime ();
extern void prtime ();

/* from debug.c */
extern int cIfDebug ();
extern void cSetDebug ();

/* from die.c */
extern void die ();

/* from fopen.c */
extern FILE * cfopen ();

/* from getopt.c */
extern int getopt ();

/* from grow.c */
extern char *grow ();

/* from monit.c */
extern startmonitime ();
extern monitime ();
extern stopmonitime ();

/* from monitor.c */
extern monitorStart ();
extern monitorStop ();

/* from mprintf.c */
extern char *mprintf ();

/* from new.c */
extern char *new ();
extern char *resize ();
extern char *new_p ();
extern char *resize_p ();
extern dispose ();
# ifdef MSTAT
extern mstat ();
# endif MSTAT

/* from param.c */
extern void paramError ();
extern void paramSetVerbose ();
extern void paramReadFile ();
extern void paramSet ();
extern void paramSetOption ();
extern void paramFree ();
extern char *paramLookupS ();
extern double paramLookupD ();
extern bool_t paramLookupB ();

/* from say.c */
extern void say ();
extern void verboseSetMode ();
extern void verbose ();
extern void message ();

/* from signal.c */
extern void catchSignals ();

/* from strfind.c */
extern char *strfind ();

/* from strsave.c */
extern char *strsave ();
