/* SccsId = "@(#)config.h 4.12 (TU-Delft) 05/06/93" */
/**********************************************************

Name/Version      : config/4.12

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Authors           : A.J. van Genderen
                  : N.P. van der Meijs
Creation date     : 15-Mar-1988
Modified by       :
Modification date :


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1988. All rights reserved.
**********************************************************/
/* Configuration:
*/

# ifndef   NCF_RELEASE
# define   NCF_RELEASE 	300	/* ALWAYS include dmincl.h before this file */
# endif /* NCF_RELEASE */

/* # define MANATTAN		/* only orthogonal layout */
/* # define DOUBLE		/* Use double precision instead of ints
				   In this case, SCALE can be 1 */

# define FAST			/* Use inline macros for some functions */

# ifdef CONFIG_XSPACE
# define DEBUG         		/* Turn debugging on when defined */
# endif /* CONFIG_XSPACE */

# define ASSERT			/* Turn on run-time assertion checking */
# define INFO			/* Statistics */
/* # define MSTAT   		/* Allocation checking and statistics */
# define STITCH			/* Maintain a band of tiles */

# ifdef CONFIG_XSPACE
# define DISPLAY  		/* Animation */
# define X11	  		/* Animation using x11 */
# define CAP3D   		/* 3-d capacitance calculation */
# endif /* CONFIG_XSPACE */

# define TOR_DS_BOUNDARIES	/* Maintain drain/source boundaries
				   when extracting transistors.
				   This way, separate drains/sources 
				   are found. */

/* # define MONITOR  		/* -%m option for producing mon.out file
				   to be processed with prof (1), if your
				   system supports the monitor (3) function */

# define SCALE           4       /* Resolution of geo-coordinates, should
				 * be equal to SCALE in makegln. */
# define MAX_ELEM_NAME  20	/* Max length of element name */

# define MAXCONDUCTORS  10       /* Max number of conductor elements */

/* # define ANY_SLOPE		/* any slope geometry */

# define MES_SUPPLYSHORT         /* Give a warning when vdd is connected to
				   vss/gnd. */

/* # define DEBUG_NODES		/* Debug the nodes that are in core: use it to
				   debug when node items are left in core ! */

/* CIR_NET_ATOM_AVAILABLE identifies a capability of libddm/libcirfmt
 * for writing a net 'structure by structure'.
 */
# if NCF_RELEASE < 400
# ifdef CIR_NET_ATOM
# define CIR_NET_ATOM_AVAILABLE
# endif
# endif /* NCF_RELEASE */

