static char *SccsId = "@(#)dberror.c 4.1 (TU-Delft) 05/23/90";
/**********************************************************

Name/Version      : sls_exp/4.1

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : A.J. van Genderen
Creation date     : 10-Jul-1986
Modified by       : S. de Graaf
Modification date : 10-Jul-1986


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1986 , All rights reserved
**********************************************************/
#include "extern.h"

dberror (fn, n, str1, str2)
char * fn;
int n;
char * str1;
char * str2;
{        
    if (fn != NULL) {
	if (n >= 0)
	    fprintf (stderr, "\"%s\", item %d: ", fn, n);
	else
	    fprintf (stderr, "%s: ", fn);
    }

    if (str1 != NULL) {
	fprintf (stderr, "%s", str1);
    }

    if (str2 != NULL) {
	fprintf (stderr, " %s", str2);
    }

    fprintf (stderr ,"\n");

    die (1);
}
