static char *SccsId = "@(#)dmclpcnt.c 3.3 (TU-Delft) 03/26/91";
/**********************************************************

Name/Version      : The Design Data Management Project

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000/S800

Author(s)         : P. van der Wolf
Creation date     : 14-Nov-1989
Modified by       : 
Modification date : 


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1985-1988, All rights reserved
**********************************************************/
#include "dmstd.h"

/*ARGSUSED*/
dmCloseProjectContinue (dmproject)
DM_PROJECT * dmproject;
{
}
