static char *SccsId = "@(#)no_groups.c 4.1 (TU-Delft) 05/24/90";
/**********************************************************

Name/Version      : makevln/4.1

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : J. Annevelink
Creation date     : 18-May-1983
Modified by       : S. de Graaf
Modification date : 29-Jan-1987


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1987 , All rights reserved
**********************************************************/
#include "incl.h"

struct group_tree  *group_ptr;

number_groups ()
{
/* number the groups in the group_tree
*/
    struct group_tree  *group;
    int     group_number = 0;

    group = group_ptr;
    while (group) {
	if (group -> parent == NULL)
	    group -> tree_count = ++group_number;
	group = group -> next;
    }

    return (group_number);
}
