/* SccsId = "@(#)debug.h 4.3 (TU-Delft) 01/28/93" */
/**********************************************************

Name/Version      : aux/4.3

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Authors           : A.J. van Genderen
                  : N.P. van der Meijs
Creation date     : 15-Mar-1988
Modified by       :
Modification date :


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1988. All rights reserved.
**********************************************************/

#ifdef  DEBUG

extern int  _IfDebug;
extern int  cIfDebug ();
extern void cSetDebug ();

#undef  DEBUG
#define DEBUG 	        (_IfDebug && cIfDebug(__FILE__, __LINE__))
#define PRINT(S)	if (DEBUG) fprintf(stderr,"%s\n",S)
#define TRACE     	if (DEBUG) fprintf(stderr,"\n")
#define Debug(s)	if (DEBUG) {s;}
#define Debug2(s)	{s;}

#else

#define PRINT(S)	/* empty */
#define TRACE           /* empty */
#define Debug(s)	/* empty */
#define Debug2(s)	/* empty */

#endif /* DEBUG */
