static char *SccsId = "@(#)BWINVERT.c 4.1 (TU-Delft) 05/24/90";
/**********************************************************

Name/Version      : libfunc/4.1

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : GOULD

Author(s)         : O. Hol
Creation date     : 18-Feb-1988
Modified by       : 
Modification date : 


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1988 , All rights reserved
**********************************************************/
#include <stdio.h>
#include <varargs.h>

char * adm_bsalloc();

/* BWINVERT() : inverts bit-wise the bit string           */

char *BWINVERT (str)
char *str;
{
    char	*res;
    int		i;

    res = adm_bsalloc ( strlen(str) + 1, 'p' );

    for ( i=0; i < strlen(str) ; i++ ) {
	    if ( str[i] == 'I' )
	        res[i] = 'O';
	    else if ( str[i] == 'O' )
	        res[i] = 'I';
	    else if ( str[i] == 'X' )
	        res[i] = 'X';
    }
    res[i]='\0';
    return(res);
}

