static char *SccsId = "@(#)print_sr.c 4.1 (TU-Delft) 05/24/90";
/**********************************************************

Name/Version      : DUBCHECK/4.1

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : J. Liedorp
Creation date     : 25-Nov-1985
Modified by       : J. Liedorp
Modification date : 05-Dec-1985


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1985 , All rights reserved
**********************************************************/
#include "dubcheck.h"

print_sr () {

/* This procedure prints the contents of the stateruler. */
/* It is intended to be used with the debugger cdb.	 */

    struct sr_field *c_sr;

    printf ("\n yb  yt  x1   x2   lay1 lay2 grp1 grp2 helplay");
    c_sr = h_sr;
    while (c_sr != h_sr -> prev) {
	printf ("\n%d    %d    %d    %d    %d    %d    %d    %d",
		c_sr -> yb, c_sr -> yt, c_sr -> xstart[0], c_sr -> xstart[1],
		c_sr -> lay_status[0], c_sr -> lay_status[1],
		c_sr -> group[0], c_sr -> group[1], c_sr -> helplay_status);
	c_sr = c_sr -> next;
    }
}
