/* SccsId = "@(#)node.h 4.4 (TU-Delft) 11/24/92" */
/**********************************************************

Name/Version      : space/4.4

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : A.J. van Genderen
		  : N.P. van der Meijs
Creation date     : 15-Mar-1988
Modified by       :
Modification date :


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1988. All rights reserved.
**********************************************************/
typedef struct node {
    char type;
    char flag;
    char keep;
    struct terminal * names;
    struct subnode * subs;
    struct element * con;
    struct element * cap;
    int res_cnt;
    int cap_cnt;
    double gndCap;
    double help;
    struct nodeTorLink * ports;
    struct subnode * pin;
    struct netEquiv * netEq;
    struct group * grp;
    struct node  * gr_father;
    int    condemned;
    int    n_children;
    struct node * next;
    struct node * prev;
    int    mask;
    coor_t x;
    coor_t y;
} node_t;
