static char *SccsId = "@(#)ini.c 3.1 (TU-Delft) 03/03/87";
/**********************************************************

Name/Version      : DIMCHECK/3.1

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)	  : T.G.R. van Leuken, J.T. Fokkema
Creation date	  : 15-Jun-1982
Modified by       : J. Liedorp
Modification date : 10-Sep-1985


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1984 , All rights reserved
**********************************************************/
#include "dimcheck.h"

/* This procedure makes a table used by gap and width	 */
/* tests in which otherwise the square root of the sum   */
/* of quadrats would have to be used. This is done to	 */
/* speed up these tests.				 */

ini ()
{
    int     i, j;

    ALLOC (h_sr, sr_field);
    NALLOC (ptable, MAXGAPTABLE + 1, struct table);

    for (i = 1; i <= MAXGAPTABLE; i++)
	for (j = 0; j < i; j++)
	    ptable[i].dis[j] = (int) (sqrt ((double) (i * i) -
			(double) (j * j)) + 0.99999999);
}
