/* SccsId = "@(#)type.h 4.3 (TU-Delft) 11/24/92" */
/**********************************************************

Name/Version      : space/4.3

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : A.J. van Genderen
		  : N.P. van der Meijs
Creation date     : 15-Mar-1988
Modified by       :
Modification date :


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1988. All rights reserved.
**********************************************************/

/*
 * This file is to be included by all .c files
 * and contains type definitions that should be
 * known globally.
 */

#include "aux/bool.h"
#include "include/tile.h"
#include "include/maskinfo.h"
#include "include/node.h"
#include "include/subnode.h"
#include "include/terminal.h"
#include "include/transistor.h"
#include "include/extract.h"
#include "include/lump.h"

