static char *SccsId = "@(#)die.c 4.4 (TU-Delft) 01/28/93";
/**********************************************************

Name/Version      : aux/4.4

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Authors           : A.J. van Genderen
                  : N.P. van der Meijs
Creation date     : 15-Mar-1988
Modified by       :
Modification date :


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1988. All rights reserved.
**********************************************************/

/*
 * Default die routine, called by signal handlers defined
 * with catchSignals. Usually redefined in application,
 * to do specific clean-up.
 * This one just calls exit(1).
 */
void die ()
{
    exit (1);
}
