/*
 *	@(#)err.c 1.5 Delft University of Technology 09/10/92
 */

#include <stdio.h>

#include "prototypes.h"

void err(int exitstatus, char *mesg)
{
fprintf(stderr,"\n\n");
fprintf(stderr,mesg);
fprintf(stderr,"\n\n");
exit(exitstatus);
}


void dumpcore(void)
{
/* don't dump.... */
err(99,"..ciao...");
}
