/* SccsId = "@(#)incl.h 3.5 (TU-Delft) 01/26/89" */
/**********************************************************

Name/Version      : rmdb/3.5

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : S. de Graaf
Creation date     : 22-Feb-1984
Modified by       : S. de Graaf
Modification date : 09-Feb-1987
Modification date : 11-Apr-1988


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1987-1988, All rights reserved
**********************************************************/
#include "stdio.h"
#include "sys/types.h"

#ifdef SYSV
#include "ndir.h"
#else
#include "sys/dir.h"
#endif

#include "signal.h"
#include "dmincl.h"

#define PE fprintf(stderr,

