/*
 * SccsId = @(#)thedate.c 1.4 Delft University of Technology 09/10/92
 */

#ifndef THEDATE
#define THEDATE "<compile date not available>"
#endif

#ifndef THEHOST
#define THEHOST "<hostname not available>"
#endif

#ifndef THEMAN
#define THEMAN "Mickey Mouse"
#endif


char *thedate = THEDATE;
char *thehost = THEHOST;
char *theman = THEMAN;
