/* SccsId = "@(#)plot.h 4.3 (TU-Delft) 11/24/92" */
/**********************************************************

Name/Version      : mplot/4.3

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author            : N.P. van der Meijs
Creation date     : 15-Mar-1988
Modified by       :
Modification date :


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1988. All rights reserved.
**********************************************************/
# include "aux/bool.h"

typedef int coor_t;

typedef struct edge {
    coor_t xl, yl, xr, yr;
    struct edge * fwd, * bwd;
    struct edge * contour, * link;
} edge_t;

#define INF      MAXTYPE (coor_t)
#define Y(e,x) \
    (e->yl==e->yr?e->yl:e->yl+((x-e->xl)*(e->yr-e->yl))/(e->xr-e->xl))

# ifdef EPSPLOT
# define LINE            0
# define TXT             1
# endif /* EPSPLOT */

