/* SccsId = "@(#)determine.h 4.6 (TU-Delft) 11/24/92" */
/**********************************************************

Name/Version      : space/4.6

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : A.J. van Genderen
		  : N.P. van der Meijs
Creation date     : 15-Mar-1988
Modified by       :
Modification date :


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1988. All rights reserved.
**********************************************************/
# define ROOT_DEPTH     1 	/* depth of root */

# define DO_EXP       001
# define DO_SPACE     002
# define DEVMOD       004
# define ISMACRO      010
# define DONE_SPACE   020

typedef struct design_object {
    char * name;	        /* cellname */
    unsigned int status;	/* 0 if not to be extracted */
    DM_CELL * lkey;
    int versionnumber;
} do_t;
