/* SccsId = "@(#)extern.h 4.8 (TU-Delft) 06/01/93" */
/**********************************************************

Name/Version      : space/4.8

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : A.J. van Genderen
		  : N.P. van der Meijs
Creation date     : 15-Mar-1988
Modified by       :
Modification date :


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1988. All rights reserved.
**********************************************************/
extern bool_t optHisto;
extern bool_t optCap;
extern bool_t optCoupCap;
extern bool_t optRes;
extern bool_t optFineNtw;
extern bool_t optNoReduc;
extern bool_t optBeeb;
extern bool_t optTorPos;
extern bool_t optNoDim;
extern bool_t optDsConJoin;
extern bool_t optInfo;
extern bool_t optPlotCir;
extern bool_t prePass1;
extern bool_t prePass2;
extern bool_t optOnlyPrePass;
extern bool_t optPrick;
extern bool_t optInvertPrick;
extern bool_t optSelectiveRes;
extern bool_t optAccWL;
extern bool_t optBackInfo;
extern bool_t optNoCore;
extern bool_t optGenNetTerm;
extern bool_t optReadNetTerm;

extern bool_t noSubnodeJoin;

extern int nrOfConductors;

extern int * conductorMask;

extern int inCap;
extern int outCap;
extern int currIntCap;
extern int maxIntCap;
extern int maxCapIfKept;
extern int inRes;
extern int outRes;
extern int currIntRes;
extern int maxIntRes;
extern int maxResIfKept;
extern int inNod;
extern int outNod;
extern int currIntNod;
extern int maxIntNod;
extern int maxIntANod;
extern int maxNodIfKept;
extern int inTor;
extern int outTor;
extern int currIntTor;
extern int maxIntTor;
extern int maxTorIfKept;
extern int inSubnod;
extern int currIntSubnod;
extern int maxIntSubnod;
extern int inSubtor;
extern int currIntSubtor;
extern int maxIntSubtor;
extern int outGrp;
extern int currIntGrp;
extern int maxIntGrp;
extern int maxIntNodGrp;
extern int maxIntResGrp;
extern int maxIntCapGrp;
extern int outPrePassGrp;
extern int currNeqv;

extern int eliNod;
extern int condemnCnt;

extern double totOutCap;

extern int * histogram;
extern int histoMaxVal;
extern int histoSize;
extern int histoBucket;

extern coor_t currX;
extern coor_t currY;

extern int currScale;
extern int tileCnt;
extern int maxTermNumber;

extern int min_art_degree;
extern int min_degree;
extern double min_res;
extern double max_par_res;
extern double min_coup_cap;
extern bool_t no_neg_res;

extern double select_res;
extern double imp_res_ratio;
extern double tor_impedance;

extern node_t  * findRoot  ();

extern int omitRes;
extern int onlyRes;
extern strRef_t * resNetList;

extern node_t * node_list;

#ifdef DEBUG_NODES
extern FILE *fp_dbn;
#endif
