/* SccsId = "@(#)config.h 4.4 (TU-Delft) 04/16/93" */
/**********************************************************

Name/Version      : config/4.4

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Authors           : A.J. van Genderen
                  : N.P. van der Meijs
Creation date     : 15-Mar-1988
Modified by       :
Modification date :


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1988. All rights reserved.
**********************************************************/
/* Configuration:
*/

/* #define MANATTAN             /* only orthogonal layout */
/* #define DEBUG                   /* Turn debugging on when defined */
/* #define ASSERT                  /* Turn on run-time assertion checking */
/* #define MSTAT                   /* Turn on memory statistics */
#define SCALE           4       /* Scale should correspond with space */

/* EOF */
