/* SccsId = "@(#)gndefine.h 4.2 (TU-Delft) 04/20/93" */
/**********************************************************

Name/Version      : sls_exp/4.2

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : A.J. van Genderen
Creation date     : 10-Jul-1986
Modified by       : S. de Graaf
Modification date : 10-Jul-1986


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1986 , All rights reserved
**********************************************************/
#define TRUE                  1
#define FALSE                 0

#define Node                 01
#define Node_t               02
#define Modelcall            03
#define Transistor           04
#define Functional           05
#define Intercap             06

#define Nenh                 01
#define Penh                 02
#define Depl                 03
#define Res                  04

#define F_OR               1001   /* the standard function numbers */
#define F_AND              1002
#define F_NOR              1003
#define F_NAND             1004
#define F_EXOR             1005
#define F_INVERT           1006
#define D_NENH             2001   /* the device numbers */
#define D_PENH             2002   
#define D_DEPL             2003
#define D_RES              2004
#define D_CAP              2005

#define NODETYPE          1
#define CHARTYPE          2
#define INTEGERTYPE       3
#define FLOATTYPE         4
#define DOUBLETYPE        5
#define EMPTYTYPE         0
#define DIFFTYPE         -1
#define NAMENEG          -2
#define NOPATH           -3
#define NONODE           -4
#define REFIERR          -5
#define REFINEG          -6
#define REFIMIS          -7
#define NOTRELEVANT      -8

#define INPUT_T           1      /* trigger output   */
#define INPUT_R           2      /* read only output */
#define OUTPUT            3
#define INOUT             7      /* bidirectional    */
#define NOTYPE            0
#define CHAR              4
#define INTEGER           5
#define FLOAT             6
#define DOUBLE            8

#define MAXDIM                4   /* maximum dimension of node or
				     modelcall (=instance) name array */
#define NAMESIZE             15   /* maximum number of characters of node,
				     model or modelcall name + 1 */
#define MAXDMPATH            80
