/* SccsId = "@(#)incl.h 4.4 (TU-Delft) 07/21/92" */
/**********************************************************

Name/Version      : xsls/4.4

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : A.J. van Genderen
Creation date     : 16-Feb-1987
Modified by       : A.J. van Genderen
Modification date : 16-Feb-1987


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1987 , All rights reserved
**********************************************************/
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dmincl.h>
#ifndef NCF_RELEASE
#define NCF_RELEASE 300
#endif
#include "define.h"
#include "type.h"
