
typedef union  {
	int   ival;
	char *sval;
	char cval;
	int *pival;
	float fval;
	double dval;
	} YYSTYPE;
extern YYSTYPE yylval;
# define INTEGER 257
# define IDENTIFIER 258
# define FLOAT 259
# define TOR_INST 260
# define NETW_INST 261
# define CAP_INST 262
# define RES_INST 263
# define T_WIDTH 264
# define T_LENGTH 265
# define SUBCKT 266
# define ENDS 267
# define END 268
# define OTHER_CARD_BEGIN 269
# define VNET 270
# define RNET 271
# define CNET 272
# define PERIOD 273
# define ASSIGN 274
# define LPS 275
# define RPS 276
# define OTHER 277
# define NEWLINE 278
