/* SccsId = "@(#)mkdbincl.h 4.4 (TU-Delft) 12/15/92" */
/**********************************************************

Name/Version      : sls_mkdb/4.4

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : A.C. de Graaf
Creation date     : 10-Jul-1986
Modified by       : S. de Graaf
Modification date : 10-Jul-1986


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1986 , All rights reserved
**********************************************************/
extern int doSimpleNet;
extern int genIntNet;
extern int externRequired;
extern int forbidFirstCapital;
extern int noWarnings;
extern int sls_errcnt;
extern int sls_errno;
extern int sls_nerr;
extern char *sls_errlist[];

extern int yylineno;
extern char fn_incl[];

extern int int_nbyte, int_maxnbyte;
extern int char_nbyte, char_maxnbyte;
extern int ntwdef_nbyte, ntwdef_maxnbyte;
extern int ntwinst_nbyte, ntwinst_maxnbyte;
extern int inst_struct_nbyte, inst_struct_maxnbyte;
extern int netelem_nbyte, netelem_maxnbyte;
extern int dict_nbyte, dict_maxnbyte;
extern int net_ref_nbyte, net_ref_maxnbyte;
extern int xelem_nbyte, xelem_maxnbyte;
extern int queue_nbyte, queue_maxnbyte;
extern int stack_nbyte, stack_maxnbyte;
