/* SccsId = "@(#)extern.h 4.5 (TU-Delft) 04/20/93" */
/**********************************************************

Name/Version      : space/4.5

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : A.J. van Genderen
		  : N.P. van der Meijs
Creation date     : 15-Mar-1988
Modified by       :
Modification date :


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1988. All rights reserved.
**********************************************************/
extern int nrOfMasks;
extern int nrOfConductors;
extern int nrOfExtConductors;

extern maskinfo_t * masktable;
extern layerCond_t * layCTab;
extern elemDef_t *** keyTab;
extern elemDef_t * elemDefTab;

extern int elemDefTabSize;

extern double * conVal;
extern double * maxResVal;

extern int * conductorMask;

extern mask_t sBitmask;
extern mask_t eBitmask;
extern mask_t oeBitmask;

extern mask_t filterBitmask;
extern mask_t torBitmask;
extern mask_t coreBitmask;

extern coor_t bandWidth;
extern coor_t effectDist;
extern int currScale;
extern int currX;
extern int currY;
extern int tileCnt;

extern subnode_t * subnGND;
extern double meters;

extern bool_t optCap;
extern bool_t optCoupCap;
extern bool_t optLatCap;
extern bool_t optRes;
extern bool_t optAllRes;
extern bool_t optVerbose;
extern bool_t optPlotCir;
extern bool_t optPrintRecog;
extern bool_t prePass1;
extern bool_t prePass2;
extern bool_t optResMesh;
extern bool_t optSelectiveRes;
extern bool_t optPrick;
extern bool_t optDsConJoin;
extern bool_t optAccWL;
extern bool_t optBackInfo;
extern bool_t optNoCore;
extern bool_t optReadNetTerm;
extern bool_t noSubnodeJoin;



extern double min_connect_width;
extern double low_sheet_res;
extern int cap_assign_type;
