/* SccsId = "@(#)assert.h 4.5 (TU-Delft) 01/28/93" */
/**********************************************************

Name/Version      : aux/4.5

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Authors           : A.J. van Genderen
                  : N.P. van der Meijs
Creation date     : 15-Mar-1988
Modified by       :
Modification date :


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1988. All rights reserved.
**********************************************************/
#ifdef  ASSERT
#undef  ASSERT
/* if condition c false, call the assert function. */
#define ASSERT(c)       if(!(c))assert(__FILE__,__LINE__)
#else
#define ASSERT(c)       /* empty */
#endif /* ASSERT */
#define CheckIndex(i,a) ASSERT (i >= 0 && i < sizeof (a) / sizeof (*a))
