static char *SccsId = "@(#)dmdebug.c 3.6 (TU-Delft) 03/26/91";
/**********************************************************

Name/Version      : The Design Data Management Project

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000/S500

Author(s)         : S. de Graaf
                    A. van der Hoeven
                    T.G.R. van Leuken
                    N. van der Meijs
		    T. Vogel
		    P. van der Wolf
Creation date     : 04-Dec-1986
Modified by       : S. de Graaf
Modification date : 26-May-1987


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1985-1987 , All rights reserved
**********************************************************/
#include "dmstd.h"

_dm_print_key (key, s)
DM_CELL * key;
char   *s;
{
#ifdef DM_DEBUG
    IFDEBUG {
	fprintf (stderr, "%s:\n", s);
	if (key == NULL)
	    fprintf (stderr, "key == NULL");
	else {
	    fprintf (stderr, "   cell: %s\n", key -> cell);
	    fprintf (stderr, " version: %s\n", key -> versionstatus);
	    fprintf (stderr, "view: %s\n", key -> view);
	    fprintf (stderr, "mode: %d\n", key -> mode);
	}
    }
#endif /* DM_DEBUG */
}
