/* SccsId = "@(#)cacdcmap.h 4.2 (TU-Delft) 03/12/91" */
/**********************************************************

Name/Version      : cacdcmap/4.2

Language          : C
Operating system  : BSD4.2 | BSD4.3 | SYS5
Host machine      : HP9000s[358]00 | GOULD PN6000 | APOLLO DN[34]000

Author(s)         : P. Bingley
Creation date     : 18-Aug-1988
Modified by       : S. de Graaf
Modification date : 17-Feb-1989


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 781708

        COPYRIGHT (C) 1988-1989, All rights reserved
**********************************************************/

#define MAXPLANES	7
#define TABSIZE		256
#define MAXINT		2147483647

#ifdef ESE
#define RGB_CACD_MAP	"ESENelsisColormap"
#else
#define RGB_CACD_MAP	"TUDNelsisCacdColormap"
#endif

#define NOTUSED		0
#define USED		1

extern char *malloc();
extern char *calloc();
extern char *realloc();

#define Malloc(type) (type *) malloc(sizeof(type))
#define Calloc(n,type) (type *) calloc((unsigned) (n), sizeof(type))
#define Realloc(p,n,t) (t *) realloc((char *) p, ((unsigned) ((n) * sizeof(t))))

typedef struct _IdElem {
    unsigned long	pixel;
    int			state;
} IdElem;
