static char *SccsId = "@(#)class.cls 4.4 (TU-Delft) 07/27/92";
/**********************************************************

Name/Version      : sls_mkdb/4.4

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : A.C. de Graaf
Creation date     : 10-Jul-1986
Modified by       : S. de Graaf
Modification date : 10-Jul-1986


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1986 , All rights reserved
**********************************************************/
#include "sys_incl.h"
#class
#include "class.h"
#include "mkdbincl.h"
#include "mkdbdefs.h"

DM_STREAM *dsp_term = NULL;
DM_STREAM *dsp_mc = NULL;
DM_STREAM *dsp_net = NULL;
DM_STREAM *dsp_sim_mc = NULL;

#include "stack.cls"
#include "queue.cls"
#include "diction.cls"
#include "ntwdefs.cls"
#include "xelem.cls"
#include "netelem.cls"
