/* SccsId = "@(#)slserr.h 4.7 (TU-Delft) 12/15/92" */
/**********************************************************

Name/Version      : sls_mkdb/4.7

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : A.C. de Graaf
Creation date     : 10-Jul-1986
Modified by       : S. de Graaf
Modification date : 10-Jul-1986


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1986 , All rights reserved
**********************************************************/
#define BADNETW		1
#define ATERM_LT_FTERM	2
#define ATERM_GT_FTERM	3
#define DNET_LT_SNET	4
#define DNET_GT_SNET	5
#define SYNTAX		6
#define ILLRANGE	7
#define TERMADEF	8
#define UNKTYPE		9
#define ILLNETWORK	10
#define INSTADEF	11
#define UNKNETWORK	12
#define UNKFUNC  	13
#define NTWAREF		14
#define UNKTERM		15
#define UNKDEVICE	16
#define DEQUIFAIL       17
#define UNKTERMDECL	18
#define ILLRANGETERMDECL 19
#define NOEXTERNDEF     20
#define USENONCAP       21
