#ifndef YYSTYPE
#define YYSTYPE int
#endif
#define	INTEGER	258
#define	SEMI	259
#define	POLYGON	260
#define	BOX	261
#define	ROTATE	262
#define	WIRE	263
#define	LAYER	264
#define	START	265
#define	FINISH	266
#define	DELETE	267
#define	CALL	268
#define	END	269
#define	TRANS	270
#define	MX	271
#define	MY	272
#define	SHORTNAME	273
#define	ROUNDFLASH	274
#define	USER	275


extern YYSTYPE yylval;
