static char *SccsId = "@(#)findCond.c 4.2 (TU-Delft) 08/28/90";
/**********************************************************

Name/Version      : tecc/4.2

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author            : A.J. van Genderen
Creation date     : 15-Mar-1988
Modified by       :
Modification date :


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1988. All rights reserved.
**********************************************************/
#include <stdio.h>
#include <dmincl.h>
#include "define.h"
#include "type.h"
#include "extern.h"

findConducTransf ()
{
    int i;

    conducCnt = 0;

    ALLOC (conducTransf, procdata -> nomasks, int);
    ALLOC (conducBackTransf, procdata -> nomasks, int);

    for (i = 0; i < procdata -> nomasks; i++) {
	conducTransf[i] = -1;
	conducBackTransf[i] = -1;
    }

    for (i = 0; i < res_cnt; i++) {

	if (conducTransf[ress[i].mask] < 0) {

	    conducTransf[ress[i].mask] = conducCnt;
	    conducBackTransf[conducCnt] = ress[i].mask;
	    conducCnt++;
	}
    }
}
