/* SccsId = "@(#)extern.h 4.11 (TU-Delft) 06/22/93" */
/**********************************************************

Name/Version      : sls/4.11

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : A.C. de Graaf, A.J. van Genderen
Creation date     : 10-Jul-1986
Modified by       : S. de Graaf
Modification date : 10-Jul-1986


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1986 , All rights reserved
**********************************************************/
#include <stdio.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/times.h>
#include <sys/stat.h>
#include "gndefine.h"
#include "define.h"
#include "dmincl.h"
#ifndef NCF_RELEASE
#define NCF_RELEASE     300
#endif /* NCF_RELEASE */
#include "gntype.h"
#include "type.h"

extern char * argv0;

extern FUNCVAR FV[];   /* cannot be *FV instead of FV[] because of GOULD */
extern FUNCDESCR FD[];
extern char * ST;
extern NAMETABLE * NT;
extern MODELTABLE * MT;
extern CONTEXTTABLE * CTT;
extern MODELCALLTABLE * MCT;
extern int * XT;
extern int * XX;
extern NODE * N;
extern int * DS;
extern CONTROL * C;
extern INTERCAP * I;
extern TRANSISTOR * T;
extern FUNCTION * F;
extern int * FI; 
extern int * FR; 
extern FUNCOUT * FO; 
extern char * FS; 

extern int FV_cnt;
extern int FD_cnt;
extern int ST_cnt;
extern int NT_cnt;
extern int MT_cnt;
extern int CTT_cnt;
extern int MCT_cnt;
extern int XT_cnt;
extern int XX_cnt;
extern int N_cnt;
extern int DS_cnt;
extern int C_cnt;
extern int T_cnt;
extern int F_cnt;
extern int I_cnt;
extern int FI_cnt;
extern int FR_cnt;
extern int FO_cnt;
extern int FS_cnt;

extern FILE * debug;

extern char * netwname;
extern char * ressuffix;
extern char * pltsuffix;
extern char * outsuffix;
extern char * fn_cmd;
extern char * fn_proc;
extern char * fn_res;
extern char * fn_plt;
extern char * fn_out;
extern char * fn_init;

extern int debugdata;
extern int debugsim;
extern int debugmem;
extern int monitoring;
extern int make_output;

extern int delaysim;
extern int outonchange;
extern int proclogic;
extern int printraces;
extern int printinfo;
extern int printdevices;
extern int printstatis;
extern int maxpagewidth;
extern int printremain;
extern int dissip;
extern int use_io_views;
extern int random_initialization;
extern int random_td_initialization;

extern float mindevtime;
extern float maxdevtime;
extern int simperiod;
extern int disperiod;

extern float sigtimeunit;
extern float outtimeunit;
extern float outtimeaccur;
extern float inttimeaccur;
extern float sigtoint;

extern int vH;
extern float vHtmp;
extern float vswitch_ext;
extern float vswitch;
extern float vminH;
extern float vmaxL;
extern float krise;
extern float kfall;

extern int maxevents;
extern int maxnvicin;
extern int maxtvicin;
extern int logic_depth;
extern int res_cnt;
extern int nodes_cnt;
extern int essnodes_cnt;
extern int inpnodes_cnt;
extern int funcoutpnodes_cnt;
extern int pnodes_cnt;
extern int plotnodes_cnt;
extern int events_cnt;
extern int simstep_cnt;
extern int timepoint_cnt;
extern int act_maxnvicin;
extern int act_maxtvicin;
extern int sigendless;
extern int * tdumps;
extern NODE_REF_LIST ** prinvert;

extern int tcurr;
extern int tbreak;
extern int tsimduration;
extern int sig_toffset;

extern int stopsim;
extern int stepdelay;

extern int fatalerror;

extern VICINITY vicin;
extern AGGLOMERATION agglom;
extern INTERRUPTMASK intrupm;

extern NODE_REF_LIST * pl_begin;
extern NODE_REF_LIST * pl_end;
extern NODE_REF_LIST * plotl_begin;
extern NODE_REF_LIST * plotl_end;
extern NODE_REF_LIST * disl_begin;
extern NODE_REF_LIST * disl_end;
extern ABSTRACT_OUTPUT * abstractl_begin;
extern ABSTRACT_OUTPUT * abstractl_end;
extern RES_PATH * rp_begin;
extern RES_PATH * rp_end;
extern HISTORY_LIST * ol_begin;
extern HISTORY_LIST * ol_end;

extern int * fsnulls;
extern FUNCTION * currf;

extern DM_CELL * stimuli_key;
extern DM_CELL * simout_key;

FILE * fopen ();
char * calloc ();
char * strcpy ();
int atoi ();
double atof ();
int strcmp ();
int strlen ();
int lstate ();
char * hiername ();
char ltoc ();
int ctol ();
