/* SccsId = "@(#)defs.h 4.4 (TU-Delft) 01/20/92" */
/**********************************************************

Name/Version      : cgi/4.4

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000/500

Author(s)         : R. Paulussen
Creation date     : 15-Dec-1987
Modified by       : S. de Graaf
Modification date : 18-Mar-1988
Modification date : 22-Aug-1988
Modification date : 22-Nov-1990 (4.02)
Modification date : 14-Jan-1991 (4.03)
Modification date : 20-Jan-1992 (4.04)


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1988, All rights reserved
***********************************************************
MADE BY:
	Delft University of Technology
	Department of Submicron Technology
	Lorentzweg 1
	DELFT
	The Netherlands
**********************************************************/
#define BLOCKSIZE       2048      /* GDS tape blocksize  */
#define D_BLOCKSIZE    (BLOCKSIZE<<1)
#define VERSION            5  /* versionnumber of GDS-format  */
#define BASE_LEN           4  /* Basic-length of all records  */
#define HEADER        0X0002
#define BGNLIB        0X0102
#define BGNSTR        0X0502
#define LIBNAME       0X0206
#define UNITS         0X0305
#define ENDLIB        0X0400
#define STRNAME       0X0606
#define ENDSTR        0X0700
#define BOUNDARY      0X0800
#define PATH          0X0900
#define SREF          0X0A00
#define AREF          0X0B00
#define LAYER         0X0D02
#define DATATYPE      0X0E02
#define WIDTH         0X0F03
#define XY            0X1003
#define ENDEL         0X1100
#define SNAME         0X1206
#define COLROW        0X1302
#define STRANS        0X1A01
#define MAG           0X1B05
#define ANGLE         0X1C05
#define PATHTYPE      0X2102
#define TAPENUM       0X3202
#define TAPECODE      0X3302
#define STRCLASS      0X3401

#define BOX 	      0x2D00
#define NODE 	      0x1500
#define TEXT 	      0x0C00
#define PROPATTR      0x2B02
#define PROPVALUE     0x2C06

#define NR_BLOCKS_L     1750   /* no. of blocks by  800 bpi/600 ft */
#define NR_BLOCKS_H     3250   /* no. of blocks by 1600 bpi/600 ft */

#define R                  4   /* if added, print also record nr.  */
#define A                  3   /* pr_exit modes: A=abort,          */
#define W                  2   /* W=warning, E=exit, I=information */
#define I                  1
#define E                  0

#define MIN_LONG  0X80000000
#define MAX_LONG  0X7FFFFFFF
#define MAX_STRLEN       512   /* maximum string length            */
#define MAX_COOR        8190   /* max. no. of polygon coord. pairs */
#define MAX_DOUBLE         2
#define MAX_GDS_LAYNR     63   /* max. layer number */

#define BUFLEN            80

#define DOUBLE             8
#define LONG	           4
#define SHORT	           2

#define DEF_ERR           -1
#define FALSE              0
#define TRUE               1

#define PE    fprintf(stderr,

