static char *SccsId = "@(#)dmdata.c 3.14 (TU-Delft) 03/26/91";
/**********************************************************

Name/Version      : The Design Data Management Project

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000/S500

Author(s)         : S. de Graaf
                    A. van der Hoeven
                    T.G.R. van Leuken
                    N. van der Meijs
		    T. Vogel
		    P. van der Wolf
Creation date     : 04-Dec-1986
Modified by       : S. de Graaf
Modification date : 11-Mar-1987
Modified by       : P. Bingley
Modification date : 15-Nov-1989


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1985-1987 , All rights reserved
**********************************************************/
#include "dmstd.h"

#if defined(hpux) || defined(MSDOS)
_dmData_ref () {
    return (0);
}
#endif 

struct geo_info ginfo;
struct geo_info2    ginfo2;
struct geo_info3    ginfo3;
struct geo_box  gbox;
struct geo_mc   gmc;
struct geo_term gterm;
struct geo_nor_ini  gnor_ini;
struct geo_nor_xy   gnor_xy;
struct geo_boxlay   gboxlay;
struct geo_termlay  gtermlay;
struct geo_vlnlay   gvlnlay;
struct geo_spec gspec;
struct geo_teq  gteq;
struct geo_tid  gtid;
struct geo_lup  glup;
struct geo_nxx_ini  gnxx_ini;
struct geo_nxx_xy   gnxx_xy;
struct geo_floc gfloc;
struct geo_gln ggln;

struct cir_mc   cmc;
struct cir_term cterm;
struct cir_net  cnet;
struct cir_sls  csls;
struct cir_info cinfo;
struct cir_graphic  cgraphic;
struct cir_swift    cswift;
struct cir_fault    cfault;

struct flp_mc    fmc;
struct flp_info   finfo;
struct flp_term  fterm;
struct flp_chan   fchan;

char    icdpath[MAXLINE];
