static char *SccsId = "@(#)aux.c 4.1 (TU-Delft) 05/24/90";
/**********************************************************

Name/Version      : makebox/4.1

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : S. de Graaf
Creation date     : 22-Apr-1986
Modified by       : S. de Graaf
Modification date : 28-Aug-1987


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1987 , All rights reserved
**********************************************************/
#include "extern.h"

#ifdef DEBUG
pr_mcelmt (ptr)
struct mc_elmt *ptr;
{
    P_E "===== struct mc_elmt: %08x ======\n", ptr);
    if (ptr) {
	P_E "parent: %08x\n", ptr -> parent);
	P_E "name: %s\n", ptr -> name);
	P_E "mtx[0-2]: %ld, %ld, %ld\n",
	    ptr -> mtx[0], ptr -> mtx[1], ptr -> mtx[2]);
	P_E "mtx[3-5]: %ld, %ld, %ld\n",
	    ptr -> mtx[3], ptr -> mtx[4], ptr -> mtx[5]);
	P_E "dx, nx, dy, ny: %ld, %ld, %ld, %ld\n",
	    ptr -> dx, ptr -> nx, ptr -> dy, ptr -> ny);
	P_E "next: %08x\n\n", ptr -> mc_next);
    }
}

pr_clist (clp)
struct clist *clp;
{
    P_E "===== struct clist: %08x ======\n", clp);
    if (clp) {
	P_E "ckey: %08x\n", clp -> ckey);
	if (clp -> ckey) {
	    P_E "cell: %s\n", clp -> ckey -> cell);
	    P_E "pkey: %s\n", clp -> ckey -> dmproject);
	}
	P_E "mc_p: %08x\n", clp -> mc_p);
	P_E "next: %08x\n\n", clp -> cl_next);
    }
}
#endif
