/* SccsId = "@(#)extern.h 4.6 (TU-Delft) 07/27/92" */
/**********************************************************

Name/Version      : sls_exp/4.6

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : A.J. van Genderen
Creation date     : 10-Jul-1986
Modified by       : S. de Graaf
Modification date : 10-Jul-1986


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1986 , All rights reserved
**********************************************************/
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "search.h"
#include "gndefine.h"
#include "define.h"
#include "dmincl.h"
#ifndef NCF_RELEASE
#define NCF_RELEASE     300
#endif /* NCF_RELEASE */
#include "gntype.h"
#include "type.h"

extern FUNCVAR * FV;
extern FUNCDESCR * FD;
extern char * ST;
extern NAMETABLE * NT;
extern MODELTABLE * MT;
extern CONTEXTTABLE * CTT;
extern MODELCALLTABLE * MCT;
extern int * XT;
extern int * XX;
extern NODE * N;
extern int * DS;
extern CONTROL * C;
extern INTERCAP * I;
extern TRANSISTOR * T;
extern FUNCTION * F;
extern int * FI; 
extern int * FR; 
extern FUNCOUT * FO; 
extern char * FS;

extern int FV_cnt;
extern int FD_cnt;
extern int ST_cnt;
extern int NT_cnt;
extern int MT_cnt;
extern int CTT_cnt;
extern int MCT_cnt;
extern int XT_cnt;
extern int XX_cnt;
extern int N_cnt;
extern int DS_cnt;
extern int C_cnt;
extern int T_cnt;
extern int F_cnt;
extern int I_cnt;
extern int FI_cnt;
extern int FR_cnt;
extern int FO_cnt;
extern int FS_cnt;

extern int FV_size;
extern int FD_size;
extern int ST_size;
extern int NT_size;
extern int MT_size;
extern int CTT_size;
extern int MCT_size;
extern int XT_size;
extern int XX_size;
extern int N_size;
extern int DS_size;
extern int C_size;
extern int T_size;
extern int F_size;
extern int I_size;
extern int FI_size;
extern int FR_size;
extern int FO_size;
extern int FS_size;

extern int mcs_NT_cnt;
extern int mcs_T_cnt;
extern int mcs_F_cnt;
extern int mcs_I_cnt;

extern FILE * debug;

extern char **globNets;
extern int *globNetsNx;
extern int globNets_cnt;

extern char * argv0;

extern DM_PROJECT *dmproject;

extern int debugdata;
extern int debugmem;
extern int monitoring;

extern int force_exp;
extern int silent;
extern int rm_noterm;
extern int capcoupling;
extern int cirflag;
extern char viewtype[BUFSIZ];

extern time_t newest_ftime;
extern time_t time_first_bin;

FILE * fopen ();
char * calloc ();
char * strcpy ();
int strcmp ();
int strlen ();
