/* SccsId = "@(#)bool.h 4.2 (TU-Delft) 01/28/93" */
/**********************************************************

Name/Version      : aux/4.2

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Authors           : A.J. van Genderen
                  : N.P. van der Meijs
Creation date     : 20-Nov-92
Modified by       :
Modification date :


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1992. All rights reserved.
**********************************************************/
# ifndef BOOL_H_INCLUDED
# define BOOL_H_INCLUDED

/*
 * Boolean type.
 * I like an enum, but X11 already defines TRUE and FALSE.
 * So, do it that way if neccessary.
 */

#ifndef TRUE
typedef enum {FALSE, TRUE} bool_t;
#else
typedef int bool_t;
#endif /* TRUE */

# endif /* BOOL_H_INCLUDED */
