static char *SccsId = "@(#)read_boxes.c 4.1 (TU-Delft) 05/24/90";
/**********************************************************

Name/Version      : makevln/4.1

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : J. Annevelink
Creation date     : 18-May-1983
Modified by       : S. de Graaf
Modification date : 11-Feb-1987


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1987 , All rights reserved
**********************************************************/
#include "incl.h"

extern struct event_rec *events;

read_boxes (fp_bxx)
DM_STREAM  *fp_bxx;
{
/* read the boxes from the "_bxx" file of the cell
*/
    register struct event_rec *event = events;

    while (dmGetDesignData (fp_bxx, GEO_BOXLAY) > 0) {
	event -> xl = gboxlay.xl;
	event -> xr = gboxlay.xr;
	event -> yb = gboxlay.yb;
	event -> yt = gboxlay.yt;
	event -> attr_no = gboxlay.chk_type;
	++event;
    }
}
