static char *SccsId = "@(#)debug.c 4.2 (TU-Delft) 08/28/90";
/**********************************************************

Name/Version      : tecc/4.2

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author            : A.J. van Genderen
Creation date     : 15-Mar-1988
Modified by       :
Modification date :


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1988. All rights reserved.
**********************************************************/
#include <stdio.h>
#include <signal.h>
#include <dmincl.h>
#include "define.h"
#include "type.h"
#include "extern.h"


printcl (cond)
struct layCondRef *cond;
{
    char * m;
    char * o;
    char * p;

    while (cond != NULL) {
	m = procdata -> mask_name[cond -> layC -> lay -> mask];
	switch (cond -> layC -> lay -> occurrence) {
	    case SURFACE:   o = "";  break;
	    case EDGE:      o = "-"; break;
	    case OTHEREDGE: o = "="; break;
	}
	p = (cond -> layC -> present ? " " : " !");
	fprintf (stderr, "%s%s%s", p, o, m);
	cond = cond -> next;
    }
    fprintf (stderr, "\n");
}
