static char *SccsId = "@(#)dmcelleq.c 3.4 (TU-Delft) 03/26/91";
/**********************************************************

Name/Version      : The Design Data Management Project

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000/S500

Author(s)         : S. de Graaf
                    A. van der Hoeven
                    T.G.R. van Leuken
                    N. van der Meijs
		    T. Vogel
		    P. van der Wolf
Creation date     : 04-Dec-1986
Modified by       : S. de Graaf
Modification date : 17-May-1988


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1985-1988, All rights reserved
**********************************************************/
#include <stdio.h>
#include <dmstd.h>

char **
_dmCellEquivalence (dmproject, dom_cell, dom_view, range_view)
DM_PROJECT *dmproject;
char *dom_cell, *dom_view, *range_view;
{
    char **celll_p;

    /* This dummy function always returns an empty (null-terminated) list */
    if ((celll_p =
	(char **) malloc ((unsigned) (1 * sizeof (char *)))) == NULL) {
	dmerrno = DME_CORE;
	dmError ("_dmCellEquivalence");
	return (NULL);
    }
    *celll_p = NULL;
    return (celll_p);
}

_dmAddCellEquivalence (dmproject, dom_cell, dom_view, range_cell, range_view)
DM_PROJECT *dmproject;
char *dom_cell, *dom_view, *range_cell, *range_view;
{
    /* dummy function, returns always success */
    return (0);
}
