/* SccsId = "@(#)mkdbdefs.h 4.3 (TU-Delft) 07/27/92" */
/**********************************************************

Name/Version      : sls_mkdb/4.3

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : A.C. de Graaf
Creation date     : 10-Jul-1986
Modified by       : S. de Graaf
Modification date : 10-Jul-1986
Modified by       : P.E. Menchen
Modification date : 16-Sep-1991


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1986,1991 All rights reserved
**********************************************************/
#include "slserr.h"
#ifdef NOSYSINCL
#define NULL		0
#endif

#define FUNCTIONAL      "functional"

#define XSTACK_SIZE	10
#define DEVTABLENGTH	10
#define STACK_OVERFLOW	0x1
#define STACK_UNDERFLOW	0

#define ALREADYDEF	1

#define QueueType	0x01
#define StackType	0x02
#define DictType	0x03
#define TermType	0x04
#define NetType		0x05
#define NetRefType	0x06
#define InstanceType	0x07
#define PmQueue		0x08
#define ImQueue		0x09

#define TransistorType	0x10
#define ResistorType	0x11
#define CapacitorType	0x12
#define NetworkType	0x13
#define FunctionType	0x14

#define DevNenh		0
#define DevPenh		1
#define DevNdep		2
#define DevRes		3
#define DevCap		4

#define XELEM		0
#define STRING		1
