static char *SccsId = "@(#)collect.c 1.1 (TU-Delft) 08/21/85";
/**********************************************************

Name/Version      : getldm/1.1

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : L.K. Regenbogen 
Creation date     : 01-Jan-1985
Modified by       : S. de Graaf
Modification date : 21-Aug-1985


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1985 , All rights reserved
**********************************************************/
#include "stdio.h"

collect (in, out)
FILE   *in, *out;
{
    char    str[BUFSIZ];
    char    model[20];
    FILE   *input;
    FILE   *openr ();

    while (fscanf (in, "%s", model) != EOF) {
	sprintf (str, "%s/layout/%s_ldm", model, model);
	input = openr (str);
	while (fgets (str, BUFSIZ, input) != NULL) {
	    fputs (str, out);
	}
	fclose (input);
    }
}
