static char *SccsId = "@(#)free_sr.c 3.2 (TU-Delft) 04/13/87";
/**********************************************************

Name/Version      : DIMCHECK/3.2

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)	  : T.G.R. van Leuken, J.T. Fokkema
Creation date	  : 15-Jun-1982
Modified by       : J. Liedorp
Modification date : 03-March-1987


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1984 , All rights reserved
**********************************************************/
#include "dimcheck.h"

/* This procedure frees the stateruler */

free_state_ruler () {
    struct sr_field *c_sr;

    c_sr = h_sr -> next;
    while (c_sr != h_sr) {
	cfree (c_sr);
	c_sr = c_sr -> next;
    }
}

free_errs (head_errs)
struct chk_err *head_errs;
{

/* this procedure frees the error linked_list		 */

    struct chk_err *c_err;
    c_err = head_errs;
    while (c_err != NULL) {
	cfree (c_err);
	c_err = c_err -> next;
    }
    head_errs = NULL;
}
