static char *SccsId = "@(#)remove.c 3.4 (TU-Delft) 03/26/91";
/**********************************************************

Name/Version      : rmdb/3.4

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : S. de Graaf
Creation date     : 22-Feb-1984
Modified by       : S. de Graaf
Modification date : 09-Feb-1987
Modification date : 11-Apr-1988


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1987-1988, All rights reserved
**********************************************************/
#include "incl.h"

extern int     sw_verb;
extern char   *argv0;

#ifdef MSDOS
_dmRmView (project, path)
DM_PROJECT *project;
char    *path;
{
#ifdef DM_DEBUG
    IFDEBUG {
	fprintf (stderr, "_dmRmView: path = '%s'\n", path);
    }
#endif /* DM_DEBUG */
    if (sw_verb)
	PE "-- %s cells\n", path);

    _dmRmDir (project, path);
}
#else
_dmRmView (project, dir)
DM_PROJECT *project;
char *dir;
{
    int status, w;

    if (sw_verb)
	PE "-- %s cells\n", dir);

    if (fork () == 0) {		/* child */
	execl ("/bin/rm", "rm", "-r", dir, 0);
	PE "%s: cannot exec: /bin/rm -r %s\n", argv0, dir);
	exit (1);
    }
    else {
	w = wait (&status);	/* parent */
    }

    if (w == -1 || status)
	errexit (-5, dir);
}
#endif
