static char *SccsId = "@(#)elim.c 4.6 (TU-Delft) 06/01/93";
/**********************************************************

Name/Version      : space/4.6

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : A.J. van Genderen
		  : N.P. van der Meijs
Creation date     : 15-Mar-1988
Modified by       :
Modification date :


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1988. All rights reserved.
**********************************************************/
#include <stdio.h>
#include "include/config.h"
#include "include/type.h"
#include "aux/aux.h"
#include "lump/define.h"
#include "lump/extern.h"

elim (n)
node_t *n;
{
    double totconval;
    element_t * con;
    element_t * con2;
    element_t * cap;
    element_t * help;
    group_t * grp;
    int histoVal;
    int b;
    int R_cnt = 0;
    int C_cnt = 0;
    int lastOfGroup = 0;

    if (n -> con == NULL) {
	lastOfGroup = 1;
	grp = Grp (n);
    }

    ASSERT (n -> netEq == NULL);
    ASSERT (n -> names == NULL);

    eliNod++;


    nodeDel (n);

    if (lastOfGroup) {
	groupDel (grp);
    }
}

