/* SccsId = "@(#)extern.h 4.1 (TU-Delft) 05/24/90" */
/**********************************************************

Name/Version      : makebox/4.1

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : J. Annevelink
Creation date     : 18-May-1983
Modified by       : S. de Graaf
Modification date : 11-Feb-1987


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1987 , All rights reserved
**********************************************************/
#include "define.h"

extern struct ctree   *celltree;
extern struct cptrl   *cptrhead;
extern struct cptrl   *cptrlast;
extern struct cptrl   *cptr;
extern struct wdw     *mc_bboxl;
extern struct wdw     **arl_ptr;
extern DM_PROCDATA *process;
extern DM_STREAM  **fp_bxx;
extern DM_STREAM   *fp_tid;
extern DM_CELL     *cellkey;
extern DM_CELL     *top_key;
extern DM_PROJECT  *project;

extern char   fn_exp[];
extern char  *argv0;
extern char   fname[];

extern long  *no_bxx;
extern long   exp_reg[4];
extern int    part_exp;
extern long   exp_depth;
extern long   usr_chlev;
extern long   term_no;
extern long   chtype;
extern long   fchtype;
extern long   maxol;
extern int    mask_no;
extern int    level;
