/* SccsId = "@(#)dmdef.h 3.6 (TU-Delft) 04/28/93" */
/**********************************************************

Name/Version      : The Design Data Management Project

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000/S500

Author(s)         : S. de Graaf
                    A. van der Hoeven
                    T.G.R. van Leuken
                    N. van der Meijs
		    T. Vogel
		    P. van der Wolf
Creation date     : 04-Dec-1986
Modified by       : 
Modification date : 


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1985-1987 , All rights reserved
**********************************************************/

#ifndef __DMDEF_H
#define __DMDEF_H


/* a large value for MAXLINE. It is mostly used for pathnames
 * (but not always) and there is no checking for overflow
 * of arrays constructed using MAXLINE.
 */
#define 	MAXLINE		1024

#define	Abs(a)		((a) < 0 ? -(a) : (a))
#define Min(a,b)	((a) < (b) ? (a) : (b))
#define	Max(a,b)	((a) > (b) ? (a) : (b))
#define Round(x)	(((x) > 0) ? ((x)+0.5) : ((x)-0.5))

#endif /* __DMDEF_H */

