static char *SccsId = "@(#)BSUNDEF.c 4.1 (TU-Delft) 05/24/90";
/**********************************************************

Name/Version      : libfunc/4.1

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : GOULD

Author(s)         : O. Hol
Creation date     : 18-Feb-1988
Modified by       : 
Modification date : 


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1988 , All rights reserved
**********************************************************/
#include <stdio.h>

/*  BSUNDEF() : all bits of bit string get value 'X'      */

BSUNDEF (s)
char  s[];
{
    int i;

    for ( i=0 ; i < strlen(s) ; i++ ) {
        s[i] = 'X';
    }
}
