/* SccsId = "@(#)queue.h 4.2 (TU-Delft) 06/11/91" */
/**********************************************************

Name/Version      : sls_mkdb/4.2

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : A.C. de Graaf
Creation date     : 10-Jul-1986
Modified by       : S. de Graaf
Modification date : 10-Jul-1986


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1986 , All rights reserved
**********************************************************/
class link {
public:
	class link *next;
	char	type;
};

class queue : public link
{
	void		new(char);
	void		delete(void);
	class link	*tail;
	short		n;
public:
	void		reset(void);
	class link	*get(void);
	void	     	put(class link *);
	short		length(void);
	void		print(void);
	int		empty(void);
	class link	*access(int);
	class link	*first_elem(void);
	class link	*next_elem(class link *);
	class link	*last_elem(void);
	void 		append(class queue *);
};
