/* SccsId = "@(#)dmdebug.h 3.6 (TU-Delft) 04/28/93" */
/**********************************************************

Name/Version      : The Design Data Management Project

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000/S500

Author(s)         : S. de Graaf
                    A. van der Hoeven
                    T.G.R. van Leuken
                    N. van der Meijs
		    T. Vogel
		    P. van der Wolf
Creation date     : 04-Dec-1986
Modified by       : 
Modification date : 


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1985-1987 , All rights reserved
**********************************************************/
#ifndef __DMDEBUG_H
#define __DMDEBUG_H

/* #define DM_DEBUG		/* debugging is on */

#ifdef DM_DEBUG
#define IFDEBUG if (_dmIfdebug(__FILE__,__LINE__))
#define TRACE fprintf(stderr,"--trace: %s, %d\n",__FILE__,__LINE__)
#define ASSERT(v) {if (!(v)) \
{fprintf(stderr,"assertion failed, file %s, line %d\n",\
__FILE__, __LINE__); _dmFatal("assert");}}

#else /* NOT DM_DEBUG */
#define IFDEBUG if (0)
#define TRACE ;
#define ASSERT(v) ;
#endif /* DM_DEBUG */

#endif /* __DMDEBUG_H */

