#ifndef lint
#ifdef PACK
static char *SccsId = "@(#)dfmt.c 3.2 (TU-Delft) 03/26/91 (pack version)";
#else /* PACK */
static char *SccsId = "@(#)dfmt.c 3.2 (TU-Delft) 03/26/91";
#endif /* PACK */
#endif /* lint */
/**********************************************************

Name/Version      : The Design Data Management Project

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000/S800

Author(s)         : P. Bingley
Creation date     : 15-Nov-1989
Modified by       : 
Modification date : 


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1989, All rights reserved
**********************************************************/

dummy_format_function ()
{
    return (-1);
}
