static char *SccsId = "@(#)dmdesman.c 3.3 (TU-Delft) 03/26/91";
/**********************************************************

Name/Version      : The Design Data Management Project

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000/S500

Author(s)         : P. van der Wolf
		    J. Hegge

Creation date     : 21-Aug-1987
Modified by       : 
Modification date : 


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1985-1987 , All rights reserved
**********************************************************/
#include <dmstd.h>

_dmCOToDesMan (cellkey)
DM_CELL *cellkey;
{
    return (0);
}

_dmCIToDesMan (cellkey, ci_mode)
DM_CELL *cellkey;
int ci_mode;
{
    return (0);
}

_dmOSToDesMan (streamkey)
DM_STREAM *streamkey;
{
    return (0);
}

_dmFailOSToDesMan (streamkey)
DM_STREAM *streamkey;
{
    /* this simple desman should never let you retry */
    return (-1);
}

_dmCSToDesMan (streamkey, cs_mode)
DM_STREAM *streamkey;
int cs_mode;
{
    return (0);
}
