static char *SccsId = "@(#)zdeffunc.c 4.1 (TU-Delft) 05/24/90";
/**********************************************************

Name/Version      : sls/4.1

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : A.J. van Genderen
Creation date     : 7-Oct-1986
Modified by       : 
Modification date :


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1986 , All rights reserved
**********************************************************/
#include "extern.h"

FUNCDESCR FD[1];  /* size is 1 because of compiler */
FUNCVAR FV[1];

int FD_cnt = 0;
int FV_cnt = 0;

loaddf (fx, S)
int fx;
char * S;
{
    slserror (NULL, 0, ERROR2, "Undefined function in network", NULL);
}

initdf (fx, S)
int fx;
char * S;
{
    slserror (NULL, 0, ERROR2, "Undefined function in network", NULL);
}

evaldf (fx, S)
int fx;
char * S;
{
    slserror (NULL, 0, ERROR2, "Undefined function in network", NULL);
}
