static char *SccsId = "@(#)rem_rest.c 3.4 (TU-Delft) 04/11/88";
/**********************************************************

Name/Version      : rmdb/3.4

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : S. de Graaf
Creation date     : 22-Feb-1984
Modified by       : S. de Graaf
Modification date : 06-Feb-1987
Modification date : 11-Apr-1988


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1987-1988, All rights reserved
**********************************************************/

rem_rest ()
{
    unlink ("exp_dat");
}
