static char *SccsId = "@(#)strsave.c 4.5 (TU-Delft) 04/20/93";
/**********************************************************

Name/Version      : aux/4.5

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Authors           : A.J. van Genderen
                  : N.P. van der Meijs
Creation date     : 15-Mar-1988
Modified by       :
Modification date :


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1988. All rights reserved.
**********************************************************/
# include <stdio.h>
# include "aux.h"

char * strsave (s)
char * s;
/*
 * strsave (s) - save string <s> in dynamic memory.
 *
 * Calls new () and strcpy ().
 * String must be free'd with DISPOSE ();
 */
{
    return (strcpy (NEW (char, (unsigned) strlen (s) + 1), s));
}
