/* SccsId = "@(#)malloc.h 4.4 (TU-Delft) 04/20/93" */
/**********************************************************

Name/Version      : aux/4.4

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Authors           : A.J. van Genderen
                  : N.P. van der Meijs
Creation date     : 20-Nov-92
Modified by       :
Modification date :


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1992. All rights reserved.
**********************************************************/
/* Memory allocation
*/
#define DISPOSE(object) (dispose ((char *) object), object = NULL)

#ifdef MSTAT
#define NEW(type,nbr)  \
    (type *) new_p (sizeof (type), (unsigned) nbr, __FILE__, __LINE__)
#define RESIZE(p, type, nbr) \
    (type *) resize_p (p, sizeof (type), (unsigned) nbr, __FILE__, __LINE__)
# else
#define NEW(type,nbr) \
    (type *) new (sizeof (type), (unsigned) nbr)
#define RESIZE(p, type, nbr) \
    (type *) resize (p, sizeof (type), (unsigned) nbr)
# endif

# define FIT(type,nbr) mallocFit (sizeof(type), nbr)

# define GROW(type, buf, from, to) \
    ((type *) grow ((char*) buf, sizeof (type), (unsigned) from, (unsigned) to))
# define CLEAR(object, size) memset ((char *) object, 0, size)

# define M_INIT  	 0	/* collect memory allocation statistics */
# define M_PRINT         1	/* print memory allocation statistics */

# ifndef MSTAT   
# define mstat(c,v)      /* empty */
# endif /* MSTAT */
