/* SccsId = "@(#)extern.h 4.4 (TU-Delft) 01/18/91" */
/**********************************************************

Name/Version      : cldm/4.4

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : S. de Graaf
Creation date     : 11-Dec-1985
Modified by       : S. de Graaf
Modification date : 17-Dec-1987
Modification date : 20-May-1988
Modification date : 14-May-1990
Modification date : 18-Jan-1991


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1987-1988, All rights reserved
**********************************************************/
#include "incl.h"

/*
**  External declarations of CLDM
*/

extern DM_PROCDATA *process;
extern DM_PROJECT  *dmproject;
extern DM_CELL     *mod_key;
extern DM_STREAM   *fp_box;
extern DM_STREAM   *fp_info;
extern DM_STREAM   *fp_mc;
extern DM_STREAM   *fp_nor;
extern DM_STREAM   *fp_term;

extern struct stat stat_buf;
extern struct name_tree   *tree_ptr;
extern struct name_tree   *mod_tree;
extern struct name_tree   *tnam_tree;
extern struct name_tree   *inst_tree;

extern char    *argv0;
#if defined(__hpux)
extern char    *yytext;
#else
extern char    *yytext;
#endif
extern char    name_len[];
extern char    ms_name[];
extern char    mc_name[];
extern char    instance[];
extern char    terminal[];
extern char    layer[];

extern int     sfx, sfy;
extern int     lay_code;
extern int     yyleng;
extern int     yylineno;
extern int     err_cnt;
extern int     err_flag;
extern int     v_mode;
extern int     s_mode;
extern int     o_mode;
extern int     f_mode;
extern int     b_mode;
extern int     mode45;
extern int     ini_bbbox;
extern int     ini_mcbbox;
extern int     n_tok;
extern int     t_tok;
extern int     w_tok;
extern int     ms_tok;
extern int     w_dir;
extern int     w_width, w_x, w_y;
extern int     int_val[];
extern int     int_ind;
extern int     dx, nx, dy, ny;
extern int     tmp_i;

extern long    tx, ty;
extern long    bbnd_xl, bbnd_xr, bbnd_yb, bbnd_yt;
extern long    mcbb_xl, mcbb_xr, mcbb_yb, mcbb_yt;
extern long    mcNoImage_xl, mcNoImage_xr, mcNoImage_yb, mcNoImage_yt;

extern double  tmp_d, d_f;

extern long    cifunit;
