/* SccsId = "@(#)cmaperror.h 4.1 (TU-Delft) 05/24/90" */
/**********************************************************

Name/Version      : cacdcmap/4.1

Language          : C
Operating system  : BSD4.2 | BSD4.3 | SYS5
Host machine      : HP9000s[358]00 | GOULD PN6000 | APOLLO DN[34]000

Author(s)         : P. Bingley
Creation date     : 18-Aug-1988
Modified by       : S. de Graaf
Modification date : 17-Feb-1989


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 781708

        COPYRIGHT (C) 1988-1989, All rights reserved
**********************************************************/

#define NOINIT		0
#define NOCMAP		1
#define BADCMAP		2
#define NOGC		3
#define NOCORE		4
#define BADCOLOR	5
#define BADID		6
#define ARRAYTOSMALL	7
#define BADERROR	8

#define CMAPMAXERROR	8

extern	char	*cacdcmaperrors[];
