static char *SccsId = "@(#)date.c 4.1 (TU-Delft) 12/17/92";
/**********************************************************

Name/Version      : space/4.1

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : A.J. van Genderen
		  : N.P. van der Meijs
Creation date     : Dec-1992
Modified by       :
Modification date :


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1992. All rights reserved.
**********************************************************/
# include "date.h"

compileDate (s)
char * s;
{
    message ("This version of %s was compiled on %s", s, COMPILE_DATE);
}
