static char *SccsId = "@(#)dummy.c 4.50 (TU-Delft) 05/04/93";
/*
*      Copyright 1992 by Delft University of Technology (DUT),
*                      Delft, The Netherlands.
* 
*                        All Rights Reserved
* 
* Permission to use, copy, modify, and distribute this software and
* its  documentation  without  fee,  is  hereby  granted  for  non-
* commercial purposes  only,  provided  that  the  above  copyright
* notice  appear  in all copies and that both that copyright notice
* and this permission notice appear  in  supporting  documentation,
* and  that the name of DUT not be used in advertising or publicity
* pertaining to distribution  of  the  software  without  specific,
* written prior permission.
* 
* THE DUT DISCLAIMS ALL WARRANTIES WITH REGARD  TO  THIS  SOFTWARE,
* INCLUDING  ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
* IN NO EVENT SHALL THE DUT BE LIABLE FOR ANY SPECIAL, INDIRECT  OR
* CONSEQUENTIAL  DAMAGES  OR  ANY DAMAGES WHATSOEVER RESULTING FROM
* LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION  OF  CONTRACT,
* NEGLIGENCE  OR  OTHER  TORTIOUS  ACTION,  ARISING  OUT  OF  OR IN
* CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
* 
* Author(s): Paul Stravers
* 
* 
* Contact address:
* 
* 
* Dimes Design and Test Centre     phone: + 31 (15) 78 1459
* Delft University of Technology   fax:   + 31 (15) 62 3271
* P.O. Box 5053 		    email: nelsis@dutentb.et.tudelft.nl
* Feldmannweg 17, 2600 GB Delft    The Netherlands
*/



/* this file only exists to resolve some symbols in libc.a
 * that are referenced on a sun sparc when i link with -Bstatic.
 * Don't you hate it?
 */

#ifdef sun
int dlsym, dlopen, dlclose;
#else
static int someVariableToMakeTheCompilerShutItsMouth;
#endif

