static char *SccsId = "@(#)init.c 4.7 (TU-Delft) 05/14/93";
/**********************************************************

Name/Version      : space/4.7

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : A.J. van Genderen
		  : N.P. van der Meijs
Creation date     : 15-Mar-1988
Modified by       :
Modification date :


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1988. All rights reserved.
**********************************************************/
#include <stdio.h>
#include <dmincl.h>
#include "include/config.h"
#include "include/type.h"
#include "aux/aux.h"
#include "aux/plot.h"
#include "extract/define.h"
#include "extract/extern.h"
#include "lump/define.h"

mask_t resBitmask;

int currScale;

subnode_t * subnGND;

coor_t effectDist;

PLOT_KEY * cirPK;

int tileCnt;

char prickName[32];

double min_connect_width;
int cap_assign_type;

static int nextNr;
static DM_STREAM * dmsTileCon;

static int nextX;
static int nextY;
static int nextCon;
static int noNextXY;
static FILE *fpPrick;

static int helpArray[ MAXCONDUCTORS ];

initExtract (circuitKey, layoutKey, scale)
DM_CELL * circuitKey;
DM_CELL * layoutKey;
int scale;
{
    char c;
    int i;
    PLOT_KEY * plotInit ();
    double atof ();
    int atoi ();
    char * paramLookupS ();


    if (optPlotCir)
	cirPK = plotInit ("cir");

    initLump (circuitKey, layoutKey, scale);


    currScale = scale;

    effectDist = bandWidth;

    subnGND = NULL;

    tileCnt = 0;


    cap_assign_type = 0;    /* default */
#   ifdef DEBUG
    cap_assign_type = atoi (paramLookupS ("cap_assign_type", "0"));
#   endif

    min_connect_width = atof (paramLookupS ("min_connect_width", "2.0e-6"));
}

endExtract ()
{
    if (optPlotCir)
	plotEnd (cirPK);


    endLump ();
}


mergeCallBack ()  /* this routine, with a dummy contents, has to be defined */
{                 /* for out.c in the directory lump */
}
