/* SccsId = "@(#)dmerror.h 3.2 (TU-Delft) 03/26/91" */
/**********************************************************

Name/Version      : The Design Data Management Project

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000/S500

Author(s)         : P. Bingley
Creation date     : 11-May-1990
Modified by       : 
Modification date : 


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA Delft
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1990, All rights reserved
**********************************************************/

/*
** This file exists for forward compatibility with
** release 4 only. Tools may include this in rel 4 and
** still compile and run on rel 3. (In rel 3 the error
** handling stuff is in dmincl.h).
*/
