static char *SccsId = "@(#)BTINVERT.c 4.1 (TU-Delft) 05/24/90";
/**********************************************************

Name/Version      : libfunc/4.1

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : GOULD

Author(s)         : O. Hol
Creation date     : 18-Feb-1988
Modified by       : 
Modification date : 


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1988 , All rights reserved
**********************************************************/
#include <stdio.h>
#include <varargs.h>

/* BTINVERT : inverts bit    */

char BTINVERT (c)
char    c;
{
    if (c == 'I')
	return('O');
    else if (c == 'O')
	return('I');
    else if (c == 'X')
	return('X');
    else
	return(0);
}
