/* SccsId = "@(#)stack.h 4.2 (TU-Delft) 06/11/91" */
/**********************************************************

Name/Version      : sls_mkdb/4.2

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : A.C. de Graaf
Creation date     : 10-Jul-1986
Modified by       : S. de Graaf
Modification date : 10-Jul-1986


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1986 , All rights reserved
**********************************************************/
class stack {
	char	**min;
	char	**top;
	char	**max;
	void	new(short);
	void	delete(void);
public:
	int	empty();
	int	push(char *);
	char	*pop(void);
	char	*access(int);
	void	print(void);
	void	reset(void);
	class stack *copy(void);
	char	**base(void);
	char	**limit(void);
	int	length(void);
};
