static char *SccsId = "@(#)prsignal.c 4.1 (TU-Delft) 05/24/90";
/**********************************************************

Name/Version      : sls/4.1

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : A.C. de Graaf, A.J. van Genderen
Creation date     : 10-Jul-1986
Modified by       : S. de Graaf
Modification date : 10-Jul-1986


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1986 , All rights reserved
**********************************************************/
#include "extern.h"

/*

pr_signal (sgn_ptr)
SIGNALELEMENT * sgn_ptr;
{
    printf ("phead\t\tpsibling\tpchild\t\tval\tlen\n");
    prsig1 (sgn_ptr);
}

prsig1 (sgn_ptr)
SIGNALELEMENT * sgn_ptr;
{
    if (sgn_ptr)
	printf ("%u\t", sgn_ptr);
    else
	printf ("%u\t\t", sgn_ptr);
    if (sgn_ptr -> sibling)
	printf ("%u\t", sgn_ptr -> sibling);
    else
	printf ("%u\t\t", sgn_ptr -> sibling);
    if (sgn_ptr -> child)
	printf ("%u\t", sgn_ptr -> child);
    else
	printf ("%u\t\t", sgn_ptr -> child);

    printf ("%d\t%d\n", sgn_ptr -> val, sgn_ptr -> len);


    if (sgn_ptr -> child)
	prsig1 (sgn_ptr -> child);

    if (sgn_ptr -> sibling)
	prsig1 (sgn_ptr -> sibling);
}

*/
