/* SccsId = "@(#)extern.h 4.3 (TU-Delft) 05/26/92" */
/**********************************************************

Name/Version      : tecc/4.3

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author            : A.J. van Genderen
Creation date     : 15-Mar-1988
Modified by       :
Modification date :


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1988. All rights reserved.
**********************************************************/
extern struct contact cons[];
extern struct transistor tors[];
extern struct capacitance caps[];
extern struct resistance ress[];
extern struct vdimension vdms[];
extern struct shape shps[];

extern int con_cnt;
extern int tor_cnt;
extern int cap_cnt;
extern int res_cnt;
extern int vdim_cnt;
extern int shape_cnt;

extern DM_PROCDATA *procdata;

extern int *maskTransf;
extern int *maskBackTransf;

extern int conducCnt;
extern int *conducTransf;
extern int *conducBackTransf;

extern int sSlotCnt;
extern int eSlotCnt;
extern int oeSlotCnt;

extern int *sLayerWeight;
extern int *eLayerWeight;
extern int *oeLayerWeight;

extern int nbrLays;

extern int maxNbrKeys;

extern struct elemRef **keytab;
extern int nbrKeySlots;

extern struct layerRef *keylist;

double atof ();
