/* SccsId = "@(#)xelem.cls 4.2 (TU-Delft) 06/11/91" */
/**********************************************************

Name/Version      : sls_mkdb/4.2

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : A.C. de Graaf
Creation date     : 10-Jul-1986
Modified by       : S. de Graaf
Modification date : 10-Jul-1986


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1986 , All rights reserved
**********************************************************/
void    xelem.new (lb, rb)
short   lb,
        rb;
{
    left_bound = lb;
    right_bound = rb;
#ifdef DMEM
    xelem_nbyte += sizeof(class xelem);
    xelem_maxnbyte = xelem_nbyte > xelem_maxnbyte ? xelem_nbyte:xelem_maxnbyte;
#endif
}

void    xelem.delete () {
#ifdef DMEM
    xelem_nbyte -= sizeof(class xelem);
#endif
}

void    xelem.print () {
            printf ("[ Xelem ] ptr: %x, lb: %d\trb: %d\n",
	        this, left_bound, right_bound);
}
