/* SccsId = "@(#)ntwdefs.h 4.3 (TU-Delft) 10/22/92" */
/**********************************************************

Name/Version      : sls_mkdb/4.3

Language          : C
Operating system  : UNIX SYSTEM V
Host machine      : HP9000

Author(s)         : A.C. de Graaf
Creation date     : 10-Jul-1986
Modified by       : S. de Graaf
Modification date : 10-Jul-1986


        Delft University of Technology
        Department of Electrical Engineering
        Network Theory Section
        Mekelweg 4 - P.O.Box 5031
        2600 GA DELFT
        The Netherlands

        Phone : 015 - 786234

        COPYRIGHT (C) 1986 , All rights reserved
**********************************************************/
class ntwdef {
	void new(char *);
	void delete(void);
	int inst_gen_cnt;
	int net_gen_cnt;
public:
	char *genname(char);
	char *ntw_name;
	int local;
	Queue *termq;
	Queue *orig_termq;
	Queue *mcq;
	Queue *netq;
};

class instancestruct {
	void new(char *, Stack *);
	void delete(void);
public:
	char *inst_name;
	Stack *inst_construct;
	int termCnt;
	class netMember *nmem;
};

class ntwinst : public link {
	void new(class ntwdef *, Stack *);
	void delete(void);
public:
	class ntwdef *ntw;
	Stack *ntw_attr;
	class instancestruct *inst_struct;
	void to_db(void);
};


typedef class ntwdef Network;
typedef class ntwinst NetworkInstance;
typedef class instancestruct InstanceStruct;
